#!/bin/env python
# author Lukasz Tasz lukasz(@)tasz.eu
# date 9.11.2011
import sys, os
import xml.dom.minidom
import xml.dom.ext
from xml import xpath
from optparse import OptionParser
import tempfile
import filecmp 
import shutil


#================================================================================
#    Load XML File
#================================================================================
def loadXML(cfg):
    try:
        db= xml.dom.minidom.parse(cfg)
    except xml.parsers.expat.ExpatError, e:
        print e.args[0]
        sys.exit(4)

    #xml.dom.ext.PrettyPrint(db)
    return db

#================================================================================
#    genCmake
#================================================================================
def gencmake(src, project):
    srcDOM      =   xml.dom.minidom.Document()
    srcDOM      =   loadXML(src)
    
    nodes=xpath.Evaluate('//structure',srcDOM)
    
    if len(nodes)==0:
        print 'No such a node or or attribute'
        sys.exit(10)
    SRC_GEN = 'set ( '+project+'_SRC_GEN\n' 
    toBeMarkedAsGenerated = ""
    for node in nodes:
        SRC_GEN +=  os.path.abspath(os.curdir) + os.sep +  node.getAttribute('name')+'.cxx\n'
        toBeMarkedAsGenerated += 'set_property(SOURCE ' + os.path.abspath(os.curdir) + os.sep + node.getAttribute('name') +'.cxx PROPERTY GENERATED TRUE)\n'
    SRC_GEN += ')\n'
    fd, fname = tempfile.mkstemp()
    fh = open(fname, "w")
    fh.write(SRC_GEN)
    fh.write(toBeMarkedAsGenerated)
    fh.close()
    if not os.path.exists('gen.cmake'):
        shutil.copyfile(fname, 'gen.cmake')
    else:
        if not filecmp.cmp(fname,'gen.cmake'):
                shutil.copyfile(fname,'gen.cmake')
    os.unlink(fname)
#================================================================================
#    generate
#================================================================================
def generate(src):
    srcDOM      =   xml.dom.minidom.Document()
    srcDOM      =   loadXML(src)
    
    nodes=xpath.Evaluate('//structure',srcDOM)
    mainInclude=""
    mainBody=""
    if len(nodes)==0:
        print 'No such a node or or attribute'
        sys.exit(10)
    for node in nodes:
        strname=node.getAttribute('name')
        fh = open(strname+'.cxx', "w")
        fh.write('#include <iostream>\nusing namespace std;\nint '+strname+'()\n{\n   cout << "running '+strname+'" << endl;\n }\n')
        fh.close()
        fh = open(strname+'.hxx', "w")
        fh.write('using namespace std;\nint '+strname+'();\n')
        fh.close()
        mainInclude+='#include "'+strname+'.hxx"\n'
        mainBody+='  '+strname+'();\n'
    fh = open('main.cxx', "w")
    fh.write(mainInclude)
    fh.write('#include <iostream>\nusing namespace std;\nint main()\n{\n   cout << "running main" << endl;\n'+mainBody+' }\n')
    fh.close()
    
        

#================================================================================
#    Main Program
#================================================================================
def main():
    
    usage = """%prog [options] 
            
 generate source file list.
    """
    parser = OptionParser(usage=usage)
    
    parser.add_option("-s", "--src", dest="src", metavar="XML_FILE",
                  help="The input XML file")
    parser.add_option("-c", "--cmake-only", dest="cmake", action="store_true", help="gen only cmakelist")
    parser.add_option("-p", "--project", dest="project", action="store", help="project name")
    
    (options, args) = parser.parse_args()
    
    if options.src==None:
        parser.print_help()
        sys.exit(1)
    elif(options.cmake):
        doc=gencmake(options.src, options.project)
    else:
        doc=generate(options.src)

if __name__ == '__main__':
    main()
